part of passwordlezz;

typedef void AuthResultCallback(AuthResult result);

class PasswordlezzData with ChangeNotifier {
  static bool isWeb = false;

  final AuthResultCallback callback;
  final String appId;

  bool _disposed = false;
  bool _online = true;
  bool get online => _online;

  String _email = '';

  String get email => _email;

  bool get emailButton => email.isNotEmpty && emailData == null;

  String _qrCodeData;

  String get qrCodeData => _qrCodeData;

  bool get hasQrCode => qrCodeData != null;

  DateTime _startTime;

  DateTime get startTime => _startTime;

  double get timePercentage => _startTime != null
      ? DateTime.now().difference(_startTime).inMilliseconds /
          (Passwordlezz.requestTimeInSeconds * 1000)
      : 0;

  Duration get timeLeft => startTime != null
      ? Duration(seconds: Passwordlezz.requestTimeInSeconds) -
          DateTime.now().difference(_startTime)
      : Duration.zero;

  bool get hasTimeLeft => startTime != null && timeLeft != null
      ? timeLeft.inMilliseconds > 0
      : false;
  bool _isReloading = false;

  String _emailData;

  String get emailData => _emailData;
  bool _emailLoading = false;

  bool get emailLoading => _emailLoading;

  String _token;
  bool _rejected = false;

  bool get rejected => _rejected;
  bool get finished => _token != null || _rejected;
  bool _close = false;
  bool get close => _close;

  bool _callbackDone = false;

  PasswordlezzData(this.callback, this.appId) {
    load();
  }

  set email(String value) {
    _email = value;
    notifyListeners();
  }

  load() async {
    if (_disposed) return;
    dynamic d = await _httpRequest(
        'https://us-central1-passwordlezz.cloudfunctions.net/requestLogin',
        {'appId': appId});
    if (d == null) return; // Timeout
    if (_disposed) return;
    if (!d['success']) {
      _close = true;
      notifyListeners();
      return;
    }
    _qrCodeData = d['qrData'];
    reload();
  }

  loadEmailData() async {
    if (_disposed) return;
    _emailLoading = true;
    notifyListeners();
    dynamic d = await _httpRequest(
        'https://us-central1-passwordlezz.cloudfunctions.net/addEmailVerification',
        {'code': qrCodeData, 'email': email});
    if (d == null) return; // Timeout
    if (_disposed) return;
    if (!d['success']) {
      _emailLoading = false;
      notifyListeners();
      return;
    }
    _emailData = '${d['rightNumber']}';
    if (_emailData.length == 1) _emailData = "0$_emailData";
    _emailLoading = false;
    reload();
  }

  reload() async {
    if (_disposed) return;
    _startTime = DateTime.now();
    notifyListeners();
    _reloadVerification();
    await Future.delayed(Duration(seconds: Passwordlezz.requestTimeInSeconds));
    if (_disposed) return;
    notifyListeners();
  }

  _reloadVerification() async {
    if (_disposed) return;
    if (!hasTimeLeft) return;
    if (_isReloading) return;
    if (timeLeft.inMilliseconds < 3000) {
      _startTime = _startTime.subtract(Duration(seconds: 4));
      notifyListeners();
      return;
    }
    _isReloading = true;
    dynamic d = await _httpRequest(
        'https://us-central1-passwordlezz.cloudfunctions.net/requestVerification',
        {'code': qrCodeData},
        20);
    if (d == null) {
      // Timeout
      await Future.delayed(Duration(seconds: 3));
      _isReloading = false;
      _reloadVerification();
      return;
    }
    _isReloading = false;
    if (!d['success']) {
      _reloadVerification();
      return;
    }
    if (d['rejected'] != null && d['rejected']) await reject();
    await finish(d['token']);
  }

  reject() async {
    if (_disposed) return;
    _rejected = true;
    _close = true;
    _startTime = null;
    notifyListeners();
  }

  finish(String token) async {
    if (_disposed) return;
    _token = token;
    _close = true;
    _startTime = null;
    notifyListeners();
  }

  closeTransaction(BuildContext context) {
    if (_callbackDone) return;
    if (context != null) Navigator.pop(context);
    callback(AuthResult(_token, _rejected));
    _callbackDone = true;
  }

  _closeVerification() {
    return _httpRequest(
        'https://us-central1-passwordlezz.cloudfunctions.net/closeVerification',
        {'code': qrCodeData});
  }

  Future<dynamic> _httpRequest(String url,
      [Map<String, dynamic> data = const {}, int timeout = 15]) async {
    try {
      var responds =
      await http.post(url, body: data).timeout(Duration(seconds: timeout));
      if (!online && !_disposed) {
        _online = true;
        notifyListeners();
      }
      return json.decode(responds.body);
    } catch (e) {
      if (online && !_disposed) {
        _online = false;
        notifyListeners();
      }
      return null;
    }
  }

  @override
  void dispose() {
    _disposed = true;
    _closeVerification();
    closeTransaction(null);
    super.dispose();
  }
}

class AuthResult {
  final String _token;
  final bool rejected;

  AuthResult(this._token, this.rejected);

  String getToken() => _token;
}
