part of passwordlezz;

class PasswordlezzButton extends StatelessWidget {
  final AuthResultCallback callback;
  final String appId;

  final double buttonWidth;
  final double buttonHeight;
  final double buttonRadius;
  final List<Color> buttonColorGradient;
  final double buttonBorderWidth;
  final Color buttonBorderColor;
  final Color buttonTextColor;
  final FontWeight buttonTextFontWeight;
  final Color highlightColor;
  final Color splashColor;
  final double elevation;

  const PasswordlezzButton({
    Key key,
    @required this.callback,
    @required this.appId,
    this.buttonColorGradient = const [Color(0xFF4132fa), Color(0xFF61aaff)],
    this.buttonWidth = 315.0,
    this.buttonHeight = 50.0,
    this.buttonRadius = 24.0,
    this.buttonBorderWidth = 0.0,
    this.elevation = 4.0,
    this.buttonBorderColor = Colors.transparent,
    this.buttonTextColor = Colors.white,
    this.buttonTextFontWeight,
    this.highlightColor = Colors.white30,
    this.splashColor = Colors.white,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Material(
      elevation: elevation,
      shadowColor: Color(0x554132fa),
      borderRadius: BorderRadius.circular(buttonRadius + 1),
      child: Container(
        width: buttonWidth,
        height: buttonHeight,
        decoration: _getColoredBoxDecoration(context),
        child: Material(
          color: Colors.transparent,
          borderRadius: BorderRadius.circular(buttonRadius),
          child: MaterialButton(
            highlightColor: highlightColor.withOpacity(0.5),
            splashColor: splashColor.withOpacity(0.5),
            shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(buttonRadius)),
            onPressed: () =>
                Passwordlezz.authenticateWithCallback(context, callback, appId),
            child: Text(
              "Sign in Passwordlezz",
              style: TextStyle(
                color: buttonTextColor,
                fontWeight: buttonTextFontWeight != null
                    ? buttonTextFontWeight
                    : Theme.of(context).textTheme.button.fontWeight,
                fontFamily: Theme.of(context).textTheme.button.fontFamily,
                fontSize: Theme.of(context).textTheme.button.fontSize,
              ),
            ),
          ),
        ),
      ),
    );
  }

  BoxDecoration _getColoredBoxDecoration(BuildContext context) {
    // Get box decoration for transparent button.
    if (buttonColorGradient == null) {
      return BoxDecoration(
        borderRadius: BorderRadius.circular(buttonRadius),
        color: Colors.transparent,
        border: Border.all(color: buttonBorderColor, width: buttonBorderWidth),
      );
    }

    // Get box decoration for button filled with color.
    return BoxDecoration(
      borderRadius: BorderRadius.circular(buttonRadius),
      gradient: LinearGradient(
          colors: buttonColorGradient,
          begin: Alignment(-1.0, 1.0),
          end: Alignment(1.0, -1.0)),
      border: Border.all(color: buttonBorderColor, width: buttonBorderWidth),
    );
  }
}

class PasswordLezzAuthenticator extends StatefulWidget {
  final AuthResultCallback callback;
  final String appId;
  final bool flat;

  const PasswordLezzAuthenticator(
      {Key key, this.callback, this.appId, this.flat = false})
      : super(key: key);

  @override
  _PasswordLezzAuthenticatorState createState() =>
      _PasswordLezzAuthenticatorState();
}

class _PasswordLezzAuthenticatorState extends State<PasswordLezzAuthenticator>
    with TickerProviderStateMixin {
  _close(PasswordlezzData data, BuildContext context) async {
    await Future.delayed(Duration(seconds: 5));
    if (!mounted) return;
    data.closeTransaction(widget.flat ? null : context);
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () => FocusScope.of(context).requestFocus(FocusNode()),
      child: MultiProvider(
        providers: [
          ChangeNotifierProvider(
            create: (_) => PasswordlezzData(widget.callback, widget.appId),
          ),
        ],
        child: Consumer<PasswordlezzData>(builder: (context, snapshot, child) {
          if (snapshot.close) _close(snapshot, context);
          double bottom = MediaQuery
              .of(context)
              .viewInsets
              .bottom;
          return LayoutBuilder(builder: (context, constraints) {
            bool isMobile =
            Passwordlezz.isMobile(context, constraints.maxWidth);
            return SafeArea(
              child: AnimatedPadding(
                duration: Duration(milliseconds: 500),
                curve: Curves.ease,
                padding: EdgeInsets.only(bottom: bottom + 10, top: 10),
                child: Align(
                  alignment:
                      isMobile ? Alignment.bottomCenter : Alignment.center,
                  child: SingleChildScrollView(
                    child: Column(
                      mainAxisSize: MainAxisSize.min,
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: <Widget>[
                        ConstrainedBox(
                          constraints: BoxConstraints(maxWidth: 800),
                          child: Card(
                            elevation: widget.flat ? 0 : 1,
                            color: widget.flat
                                ? Colors.transparent
                                : Theme.of(context).cardColor,
                            shape: RoundedRectangleBorder(
                                borderRadius: BorderRadius.circular(20)),
                            child: IndexedStack(
                              index: snapshot.finished ? 1 : 0,
                              children: <Widget>[
                                Column(
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  children: <Widget>[
                                    if (!widget.flat)
                                      Row(
                                        mainAxisSize: MainAxisSize.min,
                                        children: <Widget>[
                                          Expanded(
                                            child: Container(),
                                          ),
                                          CloseButton(),
                                        ],
                                      ),
                                    SizedBox(
                                      height: 10,
                                    ),
                                    if (!isMobile)
                                      Row(
                                        crossAxisAlignment:
                                            CrossAxisAlignment.start,
                                        children: <Widget>[
                                          Expanded(
                                            child: Column(
                                              mainAxisSize: MainAxisSize.min,
                                              children: <Widget>[
                                                PasswordlezzTextWidget(),
                                                EmailWidget(),
                                              ],
                                            ),
                                          ),
                                          Expanded(
                                            child: Column(
                                              crossAxisAlignment:
                                                  CrossAxisAlignment.start,
                                              mainAxisAlignment:
                                                  MainAxisAlignment.spaceEvenly,
                                              mainAxisSize: MainAxisSize.min,
                                              children: <Widget>[
                                                QrCodeWidget(),
                                                TimeLeftIndicator(),
                                              ],
                                            ),
                                          ),
                                        ],
                                      ),
                                    if (isMobile) QrCodeWidget(),
                                    if (isMobile) TimeLeftIndicator(),
                                    if (isMobile) PasswordlezzTextWidget(),
                                    if (isMobile) EmailWidget(),
                                    AnimatedSize(
                                      vsync: this,
                                      duration: Duration(milliseconds: 500),
                                      curve: Curves.ease,
                                      child: snapshot.online
                                          ? Container()
                                          : Row(
                                              mainAxisAlignment:
                                                  MainAxisAlignment.center,
                                              children: <Widget>[
                                                Padding(
                                                  padding:
                                                      const EdgeInsets.only(
                                                          left: 10,
                                                          right: 10,
                                                          bottom: 20),
                                                  child: Container(
                                                    decoration: BoxDecoration(
                                                      borderRadius:
                                                          BorderRadius.circular(
                                                              20),
                                                      color: snapshot.online
                                                          ? Colors.green
                                                          : Colors.redAccent,
                                                    ),
                                                    child: Padding(
                                                      padding: const EdgeInsets
                                                              .symmetric(
                                                          horizontal: 20,
                                                          vertical: 5),
                                                      child: Text(
                                                        snapshot.online
                                                            ? "Online"
                                                            : "Offline",
                                                        style: Theme.of(context)
                                                            .textTheme
                                                            .headline
                                                            .copyWith(
                                                                color: Colors
                                                                    .white),
                                                      ),
                                                    ),
                                                  ),
                                                ),
                                              ],
                                            ),
                                    ),
                                  ],
                                ),
                                Positioned.fill(
                                  child: Flex(
                                    direction: isMobile
                                        ? Axis.vertical
                                        : Axis.horizontal,
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    children: <Widget>[
                                      SizedBox(
                                        key: Key(
                                            'success-check-${snapshot.finished}'),
                                        height: 300,
                                        width: 300,
                                        child: FlareActor(
                                          snapshot.rejected
                                              ? 'packages/passwordlezz/assets/rive/error.flr'
                                              : "packages/passwordlezz/assets/rive/success-check.flr",
                                          animation: snapshot.rejected
                                              ? 'error'
                                              : "Untitled",
                                        ),
                                      ),
                                      Flexible(
                                        child: SuccessText(
                                          key: Key(
                                              'success-text-${snapshot.finished}'),
                                        ),
                                      ),
                                      SizedBox(
                                        height: 1,
                                      ),
                                    ],
                                  ),
                                ),
                              ],
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                ),
              ),
            );
          });
        }),
      ),
    );
  }
}

class SuccessText extends StatefulWidget {
  const SuccessText({Key key}) : super(key: key);
  @override
  _SuccessTextState createState() => _SuccessTextState();
}

class _SuccessTextState extends State<SuccessText>
    with TickerProviderStateMixin {
  AnimationController controller;
  Animation<double> animation;

  @override
  void initState() {
    super.initState();
    prepareAnimations();
  }

  ///Setting up the animation
  void prepareAnimations() {
    controller = AnimationController(
        vsync: this, duration: Duration(milliseconds: 2500));
    animation = CurvedAnimation(
      curve: Interval(
        0.5,
        1,
        curve: Curves.ease,
      ),
      parent: controller,
    );
    controller.forward();
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    PasswordlezzData data = Provider.of(context);
    bool rejected = data.rejected;
    return LayoutBuilder(builder: (context, constraints) {
      bool isMobile = Passwordlezz.isMobile(context, constraints.maxWidth);
      return AnimatedBuilder(
        animation: controller,
        child: Text(
          rejected
              ? 'Login has been unsuccessful!'
              : 'You have successfully logged in!',
          style: TextStyle(
              color: rejected ? Color(0xFFD93E47) : Color(0xFF00CA71),
              fontSize: 28),
          textAlign: TextAlign.center,
        ),
        builder: (ctx, child) {
          return Sized(
            axis: isMobile ? Axis.vertical : Axis.horizontal,
            sizeFactor: animation.value,
            child: Center(
              child: Padding(
                padding: EdgeInsets.all(animation.value * 20.0),
                child: child,
              ),
            ),
          );
        },
      );
    });
  }
}

class PasswordlezzTextWidget extends StatelessWidget {
  const PasswordlezzTextWidget({
    Key key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Padding(
          padding: const EdgeInsets.only(left: 20, right: 20, top: 10),
          child: Text(
            "Passwordlezz Login",
            style: Theme.of(context).textTheme.display1,
          ),
        ),
        Padding(
          padding: const EdgeInsets.only(left: 20, right: 15, bottom: 20),
          child: Text(
            "Scan the qr-code with your phone or fill in your email.",
            style: Theme
                .of(context)
                .textTheme
                .subhead
                .copyWith(color: Colors.grey.withOpacity(0.9)),
          ),
        ),
      ],
    );
  }
}

class QrCodeWidget extends StatelessWidget {
  const QrCodeWidget({
    Key key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    PasswordlezzData data = Provider.of(context);
    bool isDark = Theme.of(context).brightness == Brightness.dark;
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        SizedBox(
          key: UniqueKey(),
          height: 120,
          width: 120,
          child: FlareActor(
            "packages/passwordlezz/assets/rive/qr-loading.flr",
            animation: data.hasQrCode ? "scanning" : "loading",
            controller:
            FlareColorController(isDark ? Colors.white : Color(0xFF506cfc)),
          ),
        ),
        Flexible(
          child: ExpandedSection(
            expand: data.hasQrCode,
            axis: Axis.horizontal,
            child: AnimatedBlurWidget(
              child: QrImage(
                data: data.qrCodeData ?? '123456789',
                version: QrVersions.auto,
                size: 200,
                foregroundColor: isDark ? Colors.white : Color(0xFF506cfc),
              ),
              blurred: data.startTime != null && !data.hasTimeLeft,
            ),
          ),
        ),
      ],
    );
  }
}

class TimeLeftIndicator extends StatefulWidget {
  @override
  _TimeLeftIndicatorState createState() => _TimeLeftIndicatorState();
}

class _TimeLeftIndicatorState extends State<TimeLeftIndicator>
    with TickerProviderStateMixin {
  AnimationController controller;
  Animation<Color> barColor;
  Animation<Color> barColor2;

  Color getColor(PasswordlezzData data) {
    if (data.hasTimeLeft)
      return controller.value > 0.5 ? barColor2.value : barColor.value;
    return Colors.redAccent;
  }

  @override
  void initState() {
    controller = AnimationController(
        duration: Duration(seconds: Passwordlezz.requestTimeInSeconds),
        vsync: this);
    barColor = ColorTween(begin: Colors.greenAccent, end: Colors.yellow)
        .animate(CurvedAnimation(parent: controller, curve: Interval(0, 0.5)));
    barColor2 = ColorTween(begin: Colors.yellow, end: Colors.redAccent)
        .animate(CurvedAnimation(parent: controller, curve: Interval(0.5, 1)));
    WidgetsBinding.instance.addObserver(
      LifecycleEventHandler(
          resumeCallBack: () =>
              Provider.of<PasswordlezzData>(context).reload()),
    );
    super.initState();
  }

  @override
  void dispose() {
    controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    PasswordlezzData data = Provider.of(context);
    if (!controller.isAnimating &&
        data.startTime != null &&
        data.timePercentage < 1) controller.forward(from: 0);
    if (controller.isAnimating &&
        data.startTime != null &&
        data.timePercentage < 1) controller.forward(from: data.timePercentage);
    return ExpandedSection(
      expand: data.startTime != null,
      child: AnimatedBuilder(
        animation: controller,
        builder: (ctx, _) {
          return InkWell(
            onTap: () => data.reload(),
            child: Padding(
              padding: const EdgeInsets.symmetric(horizontal: 10, vertical: 10),
              child: Column(
                mainAxisSize: MainAxisSize.min,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  LinearPercentIndicator(
                    lineHeight: 10,
                    progressColor: getColor(data),
                    backgroundColor: Colors.grey.withOpacity(0.1),
                    percent: min(1, data.timePercentage),
                  ),
                  LayoutBuilder(builder: (context, constraints) {
                    bool isMobile =
                    Passwordlezz.isMobile(context, constraints.maxWidth);
                    return Padding(
                      padding: EdgeInsets.only(
                          left: 10, right: 10, bottom: isMobile ? 0 : 20),
                      child: Row(
                        mainAxisSize: MainAxisSize.min,
                        children: <Widget>[
                          Flexible(
                            child: AnimatedSize(
                              duration: Duration(milliseconds: 500),
                              vsync: this,
                              curve: Curves.ease,
                              child: Text(
                                data.hasTimeLeft
                                    ? '${data.timeLeft?.inSeconds ??
                                    0} seconds left to scan the qr-code'
                                    : 'Click to refresh time',
                                style: TextStyle(
                                    color: getColor(data),
                                    fontSize: 20,
                                    fontWeight: FontWeight.bold),
                              ),
                            ),
                          ),
                          if (!data.hasTimeLeft)
                            Icon(
                              Icons.refresh,
                              color: getColor(data),
                            ),
                        ],
                      ),
                    );
                  }),
                ],
              ),
            ),
          );
        },
      ),
    );
  }
}

class EmailWidget extends StatefulWidget {
  const EmailWidget({
    Key key,
  }) : super(key: key);

  @override
  _EmailWidgetState createState() => _EmailWidgetState();
}

class _EmailWidgetState extends State<EmailWidget>
    with TickerProviderStateMixin {
  FocusNode focusNode;
  GlobalKey _globalKey = GlobalKey();

  _showHideButton(String v, PasswordlezzData data) {
    bool p = data.emailButton;
    data.email = v;
    if (data.emailButton && !p) _ensureVisible();
  }

  _ensureVisible() async {
    Timer timer = Timer.periodic(Duration(milliseconds: 15),
            (t) => Scrollable.ensureVisible(_globalKey.currentContext));
    await Future.delayed(Duration(milliseconds: 500));
    timer.cancel();
  }

  @override
  void initState() {
    focusNode = FocusNode();
    focusNode.addListener(() => _ensureVisible());
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    PasswordlezzData data = Provider.of(context);
    bool isDark = Theme.of(context).brightness == Brightness.dark;
    bool hasData = data.emailData != null;
    bool blur = hasData && !data.hasTimeLeft;
    return AnimatedPadding(
      duration: Duration(milliseconds: 500),
      curve: Curves.ease,
      padding: EdgeInsets.only(
          left: 20,
          right: 20,
          bottom: MediaQuery
              .of(context)
              .viewInsets
              .bottom == 0 ? 40 : 10),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[
          Flexible(
            child: AnimatedBlurWidget(
              blurred: blur,
              child: Padding(
                padding: const EdgeInsets.all(8.0),
                child: Material(
                  animationDuration: Duration(milliseconds: 500),
                  shape: RoundedRectangleBorder(
                      borderRadius: BorderRadius.circular(30)),
                  elevation: 3,
                  color: isDark
                      ? hasData ? Colors.white : Colors.grey[850]
                      : hasData ? Color(0xFF506cfc) : Colors.grey[100],
                  child: AnimatedSize(
                    duration: Duration(seconds: 1),
                    vsync: this,
                    curve: Curves.ease,
                    child: hasData
                        ? Padding(
                      padding: const EdgeInsets.symmetric(
                          horizontal: 12, vertical: 10),
                      child: Text(
                        data.emailData,
                        style: Theme
                            .of(context)
                            .textTheme
                            .title
                            .copyWith(
                            fontSize: 26,
                            color: isDark
                                ? Color(0xFF506cfc)
                                : Colors.white),
                      ),
                    )
                        : TextField(
                      focusNode: focusNode,
                      enabled: !data.emailLoading,
                      keyboardType: TextInputType.emailAddress,
                      decoration: InputDecoration(
                        hintText: "Email",
                        prefixIcon: Icon(
                          Icons.email,
                          color:
                          isDark ? Colors.white : Color(0xFF506cfc),
                        ),
                        fillColor:
                        isDark ? Colors.grey[850] : Colors.grey[100],
                        filled: true,
                        hintStyle: TextStyle(
                            color: isDark
                                ? Colors.white
                                : Color(0xFF506cfc)),
                        enabledBorder: OutlineInputBorder(
                            borderSide:
                            BorderSide(style: BorderStyle.none),
                            borderRadius: BorderRadius.circular(30)),
                        disabledBorder: OutlineInputBorder(
                            borderSide:
                            BorderSide(style: BorderStyle.none),
                            borderRadius: BorderRadius.circular(30)),
                        focusedBorder: OutlineInputBorder(
                            borderSide:
                            BorderSide(style: BorderStyle.none),
                            borderRadius: BorderRadius.circular(30)),
                        errorBorder: OutlineInputBorder(
                            borderSide: BorderSide(
                                style: BorderStyle.solid,
                                color: Colors.redAccent),
                            borderRadius: BorderRadius.circular(30)),
                      ),
                      onChanged: (s) => _showHideButton(s, data),
                    ),
                  ),
                ),
              ),
            ),
          ),
          SizedBox(
            width: 5,
            height: 5,
          ),
          Flexible(
            child: ExpandedSection(
              expand: data.emailButton,
              axis: Axis.vertical,
              child: Align(
                alignment: Alignment.centerRight,
                child: RaisedButton(
                  key: _globalKey,
                  onPressed: EmailValidator.validate(data.email) &&
                      !data.emailLoading &&
                      !hasData &&
                      data.qrCodeData != null
                      ? () => data.loadEmailData()
                      : null,
                  color: Color(0xFF506cfc),
                  textColor: Colors.white,
                  child: AnimatedSize(
                    duration: Duration(seconds: 1),
                    vsync: this,
                    curve: Curves.ease,
                    child: data.emailLoading || hasData
                        ? PasswordlezzData.isWeb
                        ? Text('Loading...')
                        : SizedBox(
                      width: 104,
                      height: 30,
                      child: FlareActor(
                        'packages/passwordlezz/assets/rive/loading.flr',
                        animation: 'load',
                        color: Colors.white,
                      ),
                    )
                        : Text('Use Email'),
                  ),
                  shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(30),
                  ),
                ),
              ),
            ),
          ),
          Flexible(
            child: ExpandedSection(
              expand: hasData,
              child: Center(
                child: Padding(
                  padding: const EdgeInsets.symmetric(horizontal: 10),
                  child: Text(
                    'Use this code to login via the app or use the qr-code',
                    style: Theme
                        .of(context)
                        .textTheme
                        .subtitle,
                    textAlign: TextAlign.center,
                  ),
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }
}

class FlareColorController extends FlareController {
  final Color color;

  List<FlutterColorStroke> strokes = [];

  FlareColorController(this.color);

  @override
  void initialize(FlutterActorArtboard artboard) {
    for (var node in artboard.nodes) {
      if (node is FlutterActorShape) {
        strokes.add(node.stroke);
      }
    }
  }

  @override
  bool advance(FlutterActorArtboard artboard, double elapsed) {
    for (var stroke in strokes) {
      stroke.uiColor = color;
    }
    return true;
  }

  @override
  void setViewTransform(Mat2D viewTransform) {}
}

class ExpandedSection extends StatefulWidget {
  final Widget child;
  final bool expand;
  final Axis axis;

  ExpandedSection({this.expand = false, this.child, this.axis = Axis.vertical});

  @override
  _ExpandedSectionState createState() => _ExpandedSectionState();
}

class _ExpandedSectionState extends State<ExpandedSection>
    with SingleTickerProviderStateMixin {
  AnimationController expandController;
  Animation<double> animation;

  @override
  void initState() {
    super.initState();
    prepareAnimations();
    _runExpandCheck();
  }

  ///Setting up the animation
  void prepareAnimations() {
    expandController =
        AnimationController(vsync: this, duration: Duration(milliseconds: 500));
    animation = CurvedAnimation(
      parent: expandController,
      curve: Curves.ease,
    );
  }

  void _runExpandCheck() {
    if (widget.expand) {
      expandController.forward();
    } else {
      expandController.reverse();
    }
  }

  @override
  void didUpdateWidget(ExpandedSection oldWidget) {
    super.didUpdateWidget(oldWidget);
    _runExpandCheck();
  }

  @override
  void dispose() {
    expandController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SizeTransition(
      axisAlignment: 1.0,
      sizeFactor: animation,
      child: widget.child,
      axis: widget.axis,
    );
  }
}

class Sized extends StatelessWidget {
  /// Creates a size transition.
  ///
  /// The [axis], [sizeFactor], and [axisAlignment] arguments must not be null.
  /// The [axis] argument defaults to [Axis.vertical]. The [axisAlignment]
  /// defaults to 0.0, which centers the child along the main axis during the
  /// transition.
  const Sized({
    Key key,
    this.axis = Axis.vertical,
    this.sizeFactor,
    this.axisAlignment = 0.0,
    this.child,
  })
      : assert(axis != null),
        assert(sizeFactor != null),
        assert(axisAlignment != null),
        super(key: key);

  /// [Axis.horizontal] if [sizeFactor] modifies the width, otherwise
  /// [Axis.vertical].
  final Axis axis;

  /// Number between 0 and 1.
  final double sizeFactor;

  /// Describes how to align the child along the axis that [sizeFactor] is
  /// modifying.
  ///
  /// A value of -1.0 indicates the top when [axis] is [Axis.vertical], and the
  /// start when [axis] is [Axis.horizontal]. The start is on the left when the
  /// text direction in effect is [TextDirection.ltr] and on the right when it
  /// is [TextDirection.rtl].
  ///
  /// A value of 1.0 indicates the bottom or end, depending upon the [axis].
  ///
  /// A value of 0.0 (the default) indicates the center for either [axis] value.
  final double axisAlignment;

  /// The widget below this widget in the tree.
  ///
  /// {@macro flutter.widgets.child}
  final Widget child;

  @override
  Widget build(BuildContext context) {
    AlignmentDirectional alignment;
    if (axis == Axis.vertical)
      alignment = AlignmentDirectional(-1.0, axisAlignment);
    else
      alignment = AlignmentDirectional(axisAlignment, -1.0);
    return ClipRect(
      child: Align(
        alignment: alignment,
        heightFactor: axis == Axis.vertical ? max(sizeFactor, 0.0) : null,
        widthFactor: axis == Axis.horizontal ? max(sizeFactor, 0.0) : null,
        child: child,
      ),
    );
  }
}

class AnimatedBlurWidget extends StatefulWidget {
  final Widget child;
  final bool blurred;

  const AnimatedBlurWidget({Key key, this.child, this.blurred = false})
      : super(key: key);

  @override
  _AnimatedBlurWidgetState createState() => _AnimatedBlurWidgetState();
}

class _AnimatedBlurWidgetState extends State<AnimatedBlurWidget>
    with SingleTickerProviderStateMixin {
  AnimationController blurController;
  Animation<double> animation;

  @override
  void initState() {
    super.initState();
    prepareAnimations();
    _runExpandCheck();
  }

  ///Setting up the animation
  void prepareAnimations() {
    blurController =
        AnimationController(vsync: this, duration: Duration(milliseconds: 500));
    animation = Tween<double>(begin: 0, end: 6).animate(CurvedAnimation(
      curve: Curves.ease,
      parent: blurController,
    ));
  }

  void _runExpandCheck() {
    if (widget.blurred) {
      blurController.forward();
    } else {
      blurController.reverse();
    }
  }

  @override
  void didUpdateWidget(AnimatedBlurWidget oldWidget) {
    super.didUpdateWidget(oldWidget);
    _runExpandCheck();
  }

  @override
  void dispose() {
    blurController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: blurController,
      child: widget.child,
      builder: (ctx, child) {
        return Stack(
          children: <Widget>[
            child,
            if (animation.value != 0 && !PasswordlezzData.isWeb)
              Positioned.fill(
                child: ClipRect(
                  child: BackdropFilter(
                    filter: ImageFilter.blur(
                        sigmaX: animation.value, sigmaY: animation.value),
                    child: Container(
                      color: Colors.black.withOpacity(0),
                    ),
                  ),
                ),
              ),
          ],
        );
      },
    );
  }
}

class LifecycleEventHandler extends WidgetsBindingObserver {
  LifecycleEventHandler({this.resumeCallBack, this.suspendingCallBack});

  final AsyncCallback resumeCallBack;
  final AsyncCallback suspendingCallBack;

  @override
  Future<Null> didChangeAppLifecycleState(AppLifecycleState state) async {
    switch (state) {
      case AppLifecycleState.inactive:
      case AppLifecycleState.paused:
      case AppLifecycleState.detached:
      if (suspendingCallBack != null) await suspendingCallBack();
        break;
      case AppLifecycleState.resumed:
        if (resumeCallBack != null) await resumeCallBack();
        break;
    }
  }
}
